%**************************************************************************
% This file is part of the MLS-C Library; you can redistribute it and/or 
% modify it under the terms of the GNU Lesser General Public License as 
% published by the Free Software Foundation; either version 3 of the 
% License,or any later version.
% This program is distributed in the hope that it will be useful but 
% WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
% Public License for more details.
% You should have received a copy of the GNU Lesser General Public License
% along with GNU; see the file COPYING.  If not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
% or see <http://www.gnu.org/licenses/>.
%**************************************************************************
% Created by: 
%    E Nadal, M Ouallal, JJ Rodenas, F Chinesta and FJ Fuenmayor
%  
% Release 1.0  
% Date: 18/07/2014
%**************************************************************************
%%
function [P,dPdx,dPdy]=polininterp2d(X,Y,Degree,DerivFlag)
%INPUT
    % X and Y are global coordinates
    % Degree : the polinomial degree
    % DeriveFlag: Flag used to indicate if derivatives of the polynomial
    %             respect to the global coordinates system must be
    %             evaluated.
% 
% OUTPUT
%
    % P : the polynomial
    % dPdx : Polynomial first derivative with respect to x component
    % dPdy : Polynomial first derivative with respect to y component


% Evaluate the matrix of vectors used as interpolation polinomial
if size(X,2)>size(X,1)
    X = X';
    Y = Y';
end
if nargin<4               % if ~exist('DerivFlag','var')
    DerivFlag = 0;
end

C1 = ones (length(X),1);  %Column of Ones
C0 = zeros(length(X),1);  %Column of Zeros


% Calculate the polynomial for each degree and its derivatives.
switch Degree
    case 1 % first polynomial degree
        P = [...
             C1 , ...
             X  ,Y];
        if DerivFlag
            dPdx = [...
                    C0     , ...
                    C1     , C0];
            dPdy = [...
                    C0     , ...
                    C0     , C1];
        end
    case 2 % second polynomial degree
        P = [...
             C1  , ...
             X   , Y   ,...
             X.*X, X.*Y,  Y.*Y];
        if DerivFlag
            dPdx = [...
                    C0     , ...
                    C1     , C0      , ...
                    2*X    , Y       , C0];
            dPdy = [...
                    C0     , ...
                    C0     , C1      , ...
                    C0     , X       , 2*Y];
        end
    case 3 % third polynomial degree
        X2 = X.*X;  XY=X.*Y; Y2=Y.*Y;
        P  = [...
            C1     , ...
            X      , Y     ,...
            X2     , XY    , Y2     ,...
            X2.*X  , X2.*Y , X.*Y2  , Y.*Y2];
        if DerivFlag
            dPdx = [...
                    C0     , ...
                    C1     , C0      , ...
                    2*X    , Y       , C0       , ...
                    3*X2   , 2*X.*Y  , Y2       , C0];
            dPdy = [...
                    C0     , ...
                    C0     , C1      , ...
                    C0     , X       , 2*Y      , ...
                    C0     , X2      , 2*X.*Y   , 3*Y2];
        end
    case 4 % fourth polynomial degree
        X2 = X.*X;  XY=X.*Y;   Y2=Y.*Y;
        X3 = X2.*X; X2Y=X2.*Y; XY2=X.*Y2; Y3=Y.*Y2;
        P  = [...
              C1     , ...
              X      , Y     ,...
              X2     , XY    , Y2     ,...
              X3     , X2Y   , XY2    , Y3  ,...
              X3.*X  , X3.*Y , X2.*Y2 , X.*Y3  , Y3.*Y];
        if DerivFlag
            dPdx = [...
                    C0     , ...
                    C1     , C0      , ...
                    2*X    , Y       , C0       , ...
                    3*X2   , 2*X.*Y  , Y2       , C0      , ...
                    4*X3   , 3*X2.*Y , 2*X.*Y2  , Y3      , C0    ];
            dPdy = [...
                    C0     , ...
                    C0     , C1      , ...
                    C0     , X       , 2*Y      , ...
                    C0     , X2      , 2*X.*Y   , 3*Y2    , ...
                    C0     , X3      , 2*X2.*Y  , 3*X.*Y2 , 4*Y3];
         end
    case 5 % fifth polynomial degree
        X2   = X.*X;
        XY   = X.*Y;
        Y2   = Y.*Y;
        X3   = X2.*X;
        X2Y  = X2.*Y;
        XY2  = X.*Y2;
        Y3   = Y.*Y2;
        X4   = X2.*X2;
        X3Y  = X3.*Y;
        X2Y2 = X2.*Y2;
        XY3  = X.*Y3;
        Y4   = Y3.*Y;
        P    = [...
                C1     , ...
                X      , Y     ,...
                X2     , XY    , Y2     ,...
                X3     , X2Y   , XY2    , Y3     ,...
                X4     , X3Y   , X2Y2   , XY3    , Y4     ,...
                X4.*X  , X4.*Y , X3.*Y2 , X2.*Y3 , X.*Y4  , Y4.*Y];
        if DerivFlag
            dPdx = [...
                    C0     , ...
                    C1     , C0      , ...
                    2*X    , Y       , C0       , ...
                    3*X2   , 2*X.*Y  , Y2       , C0      , ...
                    4*X3   , 3*X2.*Y , 2*X.*Y2  , Y3      , C0    , ...
                    5*X4   , 4*X3.*Y , 3*X2.*Y2 , 2*X.*Y3 , Y4    , C0 ];
            dPdy = [...
                    C0     , ...
                    C0     , C1      , ...
                    C0     , X       , 2*Y     , ...
                    C0     , X2      , 2*X.*Y  , 3*Y2     , ...
                    C0     , X3      , 2*X2.*Y , 3*X.*Y2  ,4*Y3    , ...
                    C0     , X4      , 2*X3.*Y , 3*X2.*Y2 ,4*X.*Y3 ,5*Y4];
         end
    otherwise % error displays no results for Degree > 5
        err(['Degree' num2str(Degree)...
            'not implemented in PatchMatricesUStar']);
end